"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../util");
var Printer_1 = require("../Printer");
var WellKnown_1 = require("../WellKnown");
var FieldTypes_1 = require("./FieldTypes");
var message_1 = require("./message");
var enum_1 = require("./enum");
var extensions_1 = require("./extensions");
var documentation_1 = require("./documentation");
function printFileDescriptorTSServices(fileDescriptor, exportMap) {
    if (fileDescriptor.getServiceList().length === 0) {
        return "";
    }
    var fileName = fileDescriptor.getName();
    var upToRoot = util_1.getPathToRoot(fileName);
    var printer = new Printer_1.Printer(0);
    printer.printLn("import * as grpc from \"grpc\";");
    fileDescriptor.getDependencyList().forEach(function (filePath) {
        var pseudoNamespace = util_1.filePathToPseudoNamespace(filePath);
        if (filePath in WellKnown_1.WellKnownTypesMap) {
            printer.printLn("import * as " + pseudoNamespace + " from \"" + WellKnown_1.WellKnownTypesMap[filePath] + "\";");
        }
        else {
            printer.printLn("import * as " + pseudoNamespace + " from \"" + (upToRoot + filePath) + "\";");
        }
    });
    fileDescriptor.getServiceList().forEach(function (service) {
        printer.printLn("export namespace " + service.getName() + " {");
        printer.print(documentation_1.printDocumentation('interface', service.getName() + ".Service", 1, service.getName() + " interface that provides types\n   * for methods from the given gRPC " + service.getName() + " Service."));
        printer.printIndentedLn("export interface Service {");
        var methodPrinter = new Printer_1.Printer(1);
        service.getMethodList().forEach(function (method) {
            methodPrinter.write(documentation_1.printDocumentation('method', service.getName() + ".Service." + util_1.lowercaseFirst(method.getName()), 2, service.getName() + " method declaration\n     * from the given gRPC " + service.getName() + " service."));
            var requestMessageTypeName = FieldTypes_1.getFieldType(FieldTypes_1.MESSAGE_TYPE, method.getInputType().slice(1), "", exportMap);
            var responseMessageTypeName = FieldTypes_1.getFieldType(FieldTypes_1.MESSAGE_TYPE, method.getOutputType().slice(1), "", exportMap);
            methodPrinter.printIndentedLn(util_1.lowercaseFirst(method.getName()) + "(" + util_1.getRequestType(method, requestMessageTypeName, responseMessageTypeName) + "): " + util_1.getResponseType(method, responseMessageTypeName) + ";");
        });
        printer.print(methodPrinter.output);
        printer.printIndentedLn("}");
        var configPrinter = new Printer_1.Printer(1);
        service.getMethodList().forEach(function (method) {
            configPrinter.write(documentation_1.printDocumentation('namespace', service.getName() + "." + method.getName(), 1, service.getName() + " method configuration\n   * from the given gRPC " + service.getName() + " service."));
            configPrinter.printLn("export namespace " + method.getName() + " {");
            configPrinter.printIndentedLn("export const PROTO_NAME: string = '" + fileDescriptor.getName() + "';");
            configPrinter.printIndentedLn("export const PROTO_PACKAGE: string = '" + fileDescriptor.getPackage() + "';");
            configPrinter.printIndentedLn("export const SERVICE_NAME: string = '" + service.getName() + "';");
            configPrinter.printIndentedLn("export const METHOD_NAME: string = '" + method.getName() + "';");
            configPrinter.printIndentedLn("export const REQUEST_STREAM: boolean = " + method.getClientStreaming() + ";");
            configPrinter.printIndentedLn("export const RESPONSE_STREAM: boolean = " + method.getServerStreaming() + ";");
            configPrinter.printLn("}");
        });
        printer.print(configPrinter.output);
        printer.printLn("}");
        fileDescriptor.getMessageTypeList().forEach(function (enumType) {
            printer.print(message_1.printMessage(fileName, exportMap, enumType, 0, fileDescriptor));
        });
        fileDescriptor.getExtensionList().forEach(function (extension) {
            printer.print(extensions_1.printExtension(fileName, exportMap, extension, 0));
        });
        fileDescriptor.getEnumTypeList().forEach(function (enumType) {
            printer.print(enum_1.printEnum(enumType, 0));
        });
    });
    return printer.getOutput();
}
exports.printFileDescriptorTSServices = printFileDescriptorTSServices;
//# sourceMappingURL=fileDescriptorTSServices.js.map