"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../util");
var descriptor_pb_1 = require("google-protobuf/google/protobuf/descriptor_pb");
var FieldTypes_1 = require("./FieldTypes");
var Printer_1 = require("../Printer");
var enum_1 = require("./enum");
var extensions_1 = require("./extensions");
var oneOf_1 = require("./oneOf");
var documentation_1 = require("./documentation");
function hasFieldPresence(field, fileDescriptor) {
    if (field.getLabel() === descriptor_pb_1.FieldDescriptorProto.Label.LABEL_REPEATED) {
        return false;
    }
    if (field.hasOneofIndex()) {
        return true;
    }
    if (field.getType() === FieldTypes_1.MESSAGE_TYPE) {
        return true;
    }
    if (util_1.isProto2(fileDescriptor)) {
        return true;
    }
    return false;
}
function printMessage(fileName, exportMap, messageDescriptor, indentLevel, fileDescriptor) {
    var messageName = messageDescriptor.getName();
    var messageOptions = messageDescriptor.getOptions();
    if (messageOptions !== undefined && messageOptions.getMapEntry()) {
        return "";
    }
    var objectTypeName = "AsObject";
    var toObjectType = new Printer_1.Printer(indentLevel + 1);
    toObjectType.printLn("export type " + objectTypeName + " = {");
    var printer = new Printer_1.Printer(indentLevel);
    printer.write(documentation_1.printDocumentation('interface', messageName, 0, messageName + " interface that provides properties\n * and typings from the given gRPC " + messageName + " Message."));
    printer.printLn("export interface " + messageName + " {");
    var oneOfGroups = [];
    messageDescriptor.getFieldList().forEach(function (field) {
        if (field.hasOneofIndex()) {
            var oneOfIndex = field.getOneofIndex();
            var existing = oneOfGroups[oneOfIndex];
            if (existing === undefined) {
                existing = [];
                oneOfGroups[oneOfIndex] = existing;
            }
            existing.push(field);
        }
        var snakeCaseName = field.getName().toLowerCase();
        var camelCaseName = util_1.snakeToCamel(snakeCaseName);
        var withUppercase = util_1.uppercaseFirst(camelCaseName);
        var type = field.getType();
        var exportType = FieldTypes_1.getTypeName(type);
        var fullTypeName = field.getTypeName().slice(1);
        if (type === FieldTypes_1.MESSAGE_TYPE) {
            var fieldMessageType = exportMap.getMessage(fullTypeName);
            if (fieldMessageType === undefined) {
                throw new Error("No message export for: " + fullTypeName);
            }
            if (fieldMessageType.messageOptions !== undefined && fieldMessageType.messageOptions.getMapEntry()) {
                var keyTuple = fieldMessageType.mapFieldOptions.key;
                var keyType = keyTuple[0];
                var keyTypeName = FieldTypes_1.getFieldType(keyType, keyTuple[1], fileName, exportMap);
                var valueTuple = fieldMessageType.mapFieldOptions.value;
                var valueType = valueTuple[0];
                var valueTypeName = FieldTypes_1.getFieldType(valueType, valueTuple[1], fileName, exportMap);
                if (valueType === FieldTypes_1.BYTES_TYPE) {
                    valueTypeName = "Uint8Array | string";
                }
                printer.printIndentedLn("get" + withUppercase + "Map(): jspb.Map<" + keyTypeName + ", " + valueTypeName + ">;");
                printer.printIndentedLn("clear" + withUppercase + "Map(): void;");
                toObjectType.printIndentedLn(camelCaseName + "Map: Array<[" + keyTypeName + (keyType === FieldTypes_1.MESSAGE_TYPE ? ".AsObject" : "") + ", " + valueTypeName + (valueType === FieldTypes_1.MESSAGE_TYPE ? ".AsObject" : "") + "]>,");
                return;
            }
            var withinNamespace = util_1.withinNamespaceFromExportEntry(fullTypeName, fieldMessageType);
            if (fieldMessageType.fileName === fileName) {
                exportType = withinNamespace;
            }
            else {
                exportType = util_1.filePathToPseudoNamespace(fieldMessageType.fileName) + "." + withinNamespace;
            }
        }
        var hasClearMethod = false;
        function printClearIfNotPresent() {
            if (!hasClearMethod) {
                hasClearMethod = true;
                printer.printIndentedLn("clear" + withUppercase + (field.getLabel() === descriptor_pb_1.FieldDescriptorProto.Label.LABEL_REPEATED ? "List" : "") + "(): void;");
            }
        }
        if (hasFieldPresence(field, fileDescriptor)) {
            printer.printIndentedLn("has" + withUppercase + "(): boolean;");
            printClearIfNotPresent();
        }
        var canBeUndefined = false;
        if (type === FieldTypes_1.MESSAGE_TYPE) {
            if (!util_1.isProto2(fileDescriptor) || (field.getLabel() === descriptor_pb_1.FieldDescriptorProto.Label.LABEL_OPTIONAL)) {
                canBeUndefined = true;
            }
        }
        else {
            if (util_1.isProto2(fileDescriptor)) {
                canBeUndefined = true;
            }
        }
        printer.printIndentedLn("" + camelCaseName + (canBeUndefined ? "?" : "") + ": " + exportType + (field.getLabel() === descriptor_pb_1.FieldDescriptorProto.Label.LABEL_REPEATED ? '[]' : '') + ";");
    });
    toObjectType.printLn("}");
    printer.printLn("}");
    messageDescriptor.getNestedTypeList().forEach(function (nested) {
        var msgOutput = printMessage(fileName, exportMap, nested, indentLevel, fileDescriptor);
        if (msgOutput !== "") {
            printer.write(msgOutput);
        }
    });
    messageDescriptor.getEnumTypeList().forEach(function (enumType) {
        printer.write("" + enum_1.printEnum(enumType, indentLevel));
    });
    messageDescriptor.getOneofDeclList().forEach(function (oneOfDecl, index) {
        printer.write("" + oneOf_1.printOneOfDecl(oneOfDecl, oneOfGroups[index] || [], indentLevel));
    });
    messageDescriptor.getExtensionList().forEach(function (extension) {
        printer.write(extensions_1.printExtension(fileName, exportMap, extension, indentLevel));
    });
    return printer.getOutput();
}
exports.printMessage = printMessage;
//# sourceMappingURL=message.js.map